"Use strict";
import "../../node_modules/bootstrap/dist/css/bootstrap.css";
import "../../node_modules/bootstrap-multiselect/dist/css/bootstrap-multiselect.css";
import powerbi from "powerbi-visuals-api";
import {ISelectorManager,ISelectorManagerConstructor, IFilterManager} from "../visual/visualInterfaces";
import * as d3 from "d3";
type Selection<T extends d3.BaseType> = d3.Selection<T, any, any, any>;
/*
负责页面上选择器的展示
*/
export class SelectorManager implements ISelectorManager{
    public selectorContainer : Selection<HTMLElement>;
    private selector:ISelector;
    public filterManager:IFilterManager;
    private categories: powerbi.DataViewCategoryColumn;
    private defaultSelect: powerbi.DataViewValueColumn;
    private defaultStart: powerbi.DataViewValueColumn;
    private defaultEnd: powerbi.DataViewValueColumn;
    constructor(selectorContainer:Selection<HTMLElement>,filterManager:IFilterManager){
        console.debug("selectorManager constructor start");
        this.selectorContainer=selectorContainer;
        this.filterManager=filterManager;
    }
    public switchSelector<T extends ISelector>(classSelector:new ()=>T){
        let newSelector=new classSelector();
        this.selector?.dispose();
        this.selector=newSelector;
    }
    updateData(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn){


        if(!this.selector){
            let fieldType=categories.source.type;
            if(fieldType.text){
                this.selector=new DropDownSelector(this);
            }else if(fieldType.numeric){
                this.selector=new DropDownSelector(this);
            }else if(fieldType.dateTime){
                this.selector=new CalendarSelector(this);
            }else{
                throw "only receive text/num/datetime field";
            }
        }
        

        this.selector.update(categories, defaultSelect, defaultStart, defaultEnd);
    };
    public filterStringField(selection:string[]){

    };
}

interface ISelector{
    update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn);
    dispose():void;
}

abstract class Selector implements ISelector{
    constructor(manager: ISelectorManager){
        console.debug("Abstract selector:","constructor start");
        this.manager=manager;
        this.createView();
    }
    protected abstract createView():void;
    public abstract dispose():void;
    protected readonly manager:ISelectorManager;
    public abstract update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn):void;
}
class DropDownSelector extends Selector{
    private dropDown:Selection<HTMLSelectElement>;
    protected createView() {
        console.debug('dropDownViewManager','createView start');
        this.dropDown=this.manager.selectorContainer.append("select").classed("dropDown-selector",true).classed("selector",true).attr("multiple",true);
        console.debug('dropDownViewManager','createView end');
    }
    public dispose(){
        this.dropDown.remove();
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('dropDownViewManager','update start');
        let options=this.dropDown.selectAll("option").data(categories.values,function(d){return d.toString();});//map data
        options.enter().append("option").text(function(d){return d.toString();}).attr('dropDown-option');//add
        options.exit().remove();//delete
        console.debug("dropDown:",this.dropDown);
        let filterManager:IFilterManager=this.manager.filterManager;
        // this.dropDown.on("input change",function(){
        //     console.debug('DropDownSelector:',"input change");
        //     console.debug('this',this);
        //     console.debug('filterManager:',filterManager);
        //     let selectedValues:string[]=[];
        //     for(let i=0;i<this.selectedOptions.length;i++){
        //         let option=this.selectedOptions.item(i);
        //         if(option.selected){
        //             console.debug("option selected",option);
        //             console.debug('option.text:',option.text);
        //             selectedValues.push(option.text);
        //         }
        //     }
        //     if(selectedValues.length==0){
        //     }else{
        //         console.debug("selection.length",selectedValues.length);
        //         filterManager.filterStringField(selectedValues);
        //     }
        //     console.debug("this",this.selectedOptions);
        // });
        
        $('.dropDown-selector').multiselect({
            onChange:function(){
                let selectedValues:string[]=[];
                $('option:selected').map(function(a,item){
                    selectedValues.push(item.textContent);
                    console.debug('item:',item.textContent);
                    filterManager.filterStringField(selectedValues);
                });
            },
            maxHeight:100,
            includeSelectAllOption:true
        });
        console.debug('dropDownViewManager','update end');
    }
}
class ListSelector extends Selector{
    public dispose() {
        throw new Error("Method not implemented.");
    }
    protected createView() {
        throw new Error("Method not implemented.");
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('ListSelector','updateView');
        let dropDownSelector=this.manager.selectorContainer.append("select").classed("listSelector");
    }
}
class CalendarSelector extends Selector{
    public dispose() {
        throw new Error("Method not implemented.");
    }
    protected createView() {
        throw new Error("Method not implemented.");
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('calendarViewManager','updateView');
        let dropDownSelector=this.manager.selectorContainer.append("select").classed("listSelector");
    }
}